@@ -1,6 +1,7 @@
 from hashlib import md5
 import uuid
 
+from django.contrib.auth.models import Group
 from django.core.cache import cache
 from django.db import models, transaction
 from django.contrib.postgres.fields import JSONField
@@ -15,7 +16,7 @@ from .style import generate_style_from_wizard
 
 
 class Scene(models.Model):
-    """ A scene is a group of data visualisation in terra-visu.
+    """A scene is a group of data visualisation in terra-visu.
     It's also a main menu entry.
     """
 
@@ -77,14 +78,13 @@ class Scene(models.Model):
             layer = Layer.objects.get(pk=current_node["geolayer"])
             layer.group = parent
             layer.order = order
-            layer.save()
+            layer.save(wizard_update=False)
 
     def insert_in_tree(self, layer, parts, group_config=None):
-        """ Add the layer in tree. Each parts are a group name to find inside the tree.
-            Here we assume that missing groups are added at first position of current node
-            We create missing group with default exclusive group configuration (should be corrected later if necessary)
+        """Add the layer in tree. Each parts are a group name to find inside the tree.
+        Here we assume that missing groups are added at first position of current node
+        We create missing group with default exclusive group configuration (should be corrected later if necessary)
         """
-
         group_config = group_config or {}
 
         current_node = self.tree
@@ -111,7 +111,6 @@ class Scene(models.Model):
         if group_config and last_group:
             # And update tho config
             last_group.update(group_config)
-
         self.save()
 
     def save(self, *args, **kwargs):
@@ -195,8 +194,8 @@ class Layer(models.Model):
     class Meta:
         ordering = ("order", "name")
 
-    def save(self, **kwargs):
-        if self.layer_style_wizard:
+    def save(self, wizard_update=True, **kwargs):
+        if self.layer_style_wizard and wizard_update:
             style, legend_addition = generate_style_from_wizard(
                 self, self.layer_style_wizard
             )
@@ -213,6 +212,18 @@ class Layer(models.Model):
         if self.group:
             cache.delete(get_layer_group_cache_key(self.group.view))
 
+            # deleting cache for Groups
+            groups = self.source.settings.get("groups", [])
+            for group in Group.objects.filter(id__in=groups):
+                cache.delete(
+                    get_layer_group_cache_key(
+                        self.group.view,
+                        [
+                            group.name,
+                        ],
+                    )
+                )
+
     def __str__(self):
         return f"Layer({self.id}) - {self.name}"
 
@@ -232,6 +243,7 @@ class Layer(models.Model):
                     print(f"{filter_field.field.name} replaced by {new_field.name}.")
                 else:
                     filter_field.field = new_field
+                    filter_field.save()
             else:
                 if dry_run:
                     print(f"Old field {field_name} deleted.")
