@@ -8,9 +8,30 @@ from terra_layer.settings import (
     DEFAULT_FILL_OPACITY,
     DEFAULT_STROKE_COLOR,
     DEFAULT_STROKE_WIDTH,
+    DEFAULT_CIRCLE_RADIUS,
+    DEFAULT_NO_VALUE_FILL_COLOR,
+    DEFAULT_NO_VALUE_FILL_OPACITY,
+    DEFAULT_NO_VALUE_STROKE_COLOR,
+    DEFAULT_NO_VALUE_STROKE_WIDTH,
+    DEFAULT_NO_VALUE_CIRCLE_RADIUS,
 )
 
 
+DEFAULT_STYLE = {
+    "fill_color": DEFAULT_FILL_COLOR,
+    "fill_opacity": DEFAULT_FILL_OPACITY,
+    "stroke_color": DEFAULT_STROKE_COLOR,
+    "stroke_width": DEFAULT_STROKE_WIDTH,
+}
+
+DEFAULT_STYLE_NO_VALUE = {
+    "fill_color": DEFAULT_NO_VALUE_FILL_COLOR,
+    "fill_opacity": DEFAULT_NO_VALUE_FILL_OPACITY,
+    "stroke_color": DEFAULT_NO_VALUE_STROKE_COLOR,
+    "stroke_width": DEFAULT_NO_VALUE_STROKE_WIDTH,
+    "circle_radius": DEFAULT_NO_VALUE_CIRCLE_RADIUS,
+}
+
 DEFAULT_STYLE_GRADUADED = {
     "type": "fill",
     "paint": {
@@ -20,10 +41,28 @@ DEFAULT_STYLE_GRADUADED = {
     },
 }
 
+DEFAULT_STYLE_GRADUADED_NO_VALUE = {
+    "type": "fill",
+    "paint": {
+        "fill-color": DEFAULT_NO_VALUE_FILL_COLOR,
+        "fill-opacity": DEFAULT_NO_VALUE_FILL_OPACITY,
+        "fill-outline-color": DEFAULT_NO_VALUE_STROKE_COLOR,
+    },
+}
+
+DEFAULT_LEGEND_GRADUADED = {
+    "color": DEFAULT_NO_VALUE_FILL_COLOR,
+    "boundaries": {
+        "lower": {"value": None, "included": True},
+        "upper": {"value": None, "included": True},
+    },
+    "shape": "square",
+}
+
 DEFAULT_STYLE_CIRCLE = {
     "type": "circle",
     "paint": {
-        "circle-radius": 30,
+        "circle-radius": DEFAULT_CIRCLE_RADIUS,
         "circle-color": DEFAULT_FILL_COLOR,
         "circle-opacity": DEFAULT_FILL_OPACITY,
         "circle-stroke-color": DEFAULT_STROKE_COLOR,
@@ -31,6 +70,13 @@ DEFAULT_STYLE_CIRCLE = {
     },
 }
 
+DEFAULT_LEGEND_CIRCLE = {
+    "diameter": DEFAULT_NO_VALUE_CIRCLE_RADIUS * 2,
+    "boundaries": {"lower": {"value": None}},
+    "shape": "circle",
+    "color": DEFAULT_NO_VALUE_FILL_COLOR,
+}
+
 
 def _flatten(levels):
     """
@@ -48,6 +94,7 @@ def get_min_max(geo_layer, field):
         cursor.execute(
             """
             SELECT
+                bool_or((properties->>%(field)s)::numeric IS NULL) AS is_null,
                 min((properties->>%(field)s)::numeric) AS min,
                 max((properties->>%(field)s)::numeric) AS max
             FROM
@@ -58,8 +105,8 @@ def get_min_max(geo_layer, field):
             {"field": field, "layer_id": geo_layer.id},
         )
         row = cursor.fetchone()
-        min, max = row
-        return [min, max]
+        is_null, min, max = row
+        return [is_null == True, min, max]  # noqa
 
 
 def get_positive_min_max(geo_layer, field):
@@ -70,6 +117,7 @@ def get_positive_min_max(geo_layer, field):
         cursor.execute(
             """
             SELECT
+                bool_or((properties->>%(field)s)::numeric IS NULL) AS is_null,
                 min((properties->>%(field)s)::numeric) AS min,
                 max((properties->>%(field)s)::numeric) AS max
             FROM
@@ -81,8 +129,8 @@ def get_positive_min_max(geo_layer, field):
             {"field": field, "layer_id": geo_layer.id},
         )
         row = cursor.fetchone()
-        min, max = row
-        return [min, max]
+        is_null, min, max = row
+        return [is_null == True, min, max]  # noqa
 
 
 def discretize_quantile(geo_layer, field, class_count):
@@ -99,6 +147,14 @@ def discretize_quantile(geo_layer, field, class_count):
                     ntile(%(class_count)s) OVER (ORDER BY (properties->>%(field)s)::numeric) AS ntile
                 FROM
                     geostore_feature
+                WHERE
+                    layer_id = %(layer_id)s
+                UNION ALL
+                SELECT
+                    NULL AS value,
+                    NULL AS ntile
+                FROM
+                    geostore_feature
                 WHERE
                     layer_id = %(layer_id)s AND
                     (properties->>%(field)s)::numeric IS NOT NULL
@@ -116,11 +172,13 @@ def discretize_quantile(geo_layer, field, class_count):
             {"field": field, "class_count": class_count, "layer_id": geo_layer.id},
         )
         rows = cursor.fetchall()
-        if rows and rows[0] and rows[0][0] is not None and rows[0][1] is not None:
-            # Each class start + last class end
-            return [r[0] for r in rows] + [rows[-1][1]]
-        else:
-            return []
+        if rows and len(rows) > 0:
+            rows = [row for row in rows if row[0] is not None]
+            if len(rows) == 0:
+                return []
+            else:
+                # Each class start + last class end
+                return [r[0] for r in rows] + [rows[-1][1]]
 
 
 def discretize_jenks(geo_layer, field, class_count):
@@ -142,8 +200,7 @@ def discretize_jenks(geo_layer, field, class_count):
                 FROM
                     geostore_feature
                 WHERE
-                    layer_id = %(layer_id)s AND
-                    (properties->>%(field)s)::numeric IS NOT NULL
+                    layer_id = %(layer_id)s
             )
             SELECT
                 min(field) AS min,
@@ -159,18 +216,20 @@ def discretize_jenks(geo_layer, field, class_count):
             {"field": field, "class_count": class_count, "layer_id": geo_layer.id},
         )
         rows = cursor.fetchall()
-        if rows and rows[0] and rows[0][0] is not None and rows[0][1] is not None:
-            # # Each class start + last class end
-            return [r[0] for r in rows] + [rows[-1][1]]
-        else:
-            return []
+        if rows and len(rows) > 0:
+            rows = [row for row in rows if row[0] is not None]
+            if len(rows) == 0:
+                return []
+            else:
+                # Each class start + last class end
+                return [r[0] for r in rows] + [rows[-1][1]]
 
 
 def discretize_equal_interval(geo_layer, field, class_count):
     """
     Compute QuantiEqual Interval class boundaries from a layer property.
     """
-    min, max = get_min_max(geo_layer, field)
+    is_null, min, max = get_min_max(geo_layer, field)
     if min is not None and max is not None and isinstance(min, numbers.Number):
         delta = (max - min) / class_count
         return [min + delta * i for i in range(0, class_count + 1)]
@@ -198,19 +257,42 @@ def get_field_style(field):
     return ["get", field]
 
 
+def get_style_no_value_condition(
+    include_no_value, key, with_value, with_no_value, default_value
+):
+    if include_no_value:
+        if with_value:
+            return [
+                "case",
+                ["==", ["typeof", key], "number"],
+                with_value,
+                with_no_value,
+            ]
+        else:
+            return with_no_value
+    else:
+        if with_value:
+            return with_value
+        else:
+            return default_value
+
+
 def gen_style_steps(expression, boundaries, colors):
     """
     Assume len(boundaries) <= len(colors) - 1
     """
-    return ["step", expression, colors[0]] + _flatten(zip(boundaries[1:], colors[1:]))
+    if len(boundaries) > 0:
+        return ["step", expression, colors[0]] + _flatten(
+            zip(boundaries[1:], colors[1:])
+        )
 
 
-def gen_legend_steps(boundaries, colors):
+def gen_legend_steps(boundaries, colors, include_no_value, no_value_color):
     """
     Generate a discrete legend.
     """
     size = len(boundaries) - 1
-    return [
+    ret = [
         {
             "color": colors[index],
             "boundaries": {
@@ -225,6 +307,24 @@ def gen_legend_steps(boundaries, colors):
         for index in range(size)
     ]
 
+    if include_no_value:
+        ret.insert(
+            0,
+            {
+                "color": no_value_color,
+                "boundaries": {
+                    "lower": {"value": None, "included": True},
+                    "upper": {"value": None, "included": True},
+                },
+                "shape": "square",
+            },
+        )
+
+    if not ret:
+        ret = [DEFAULT_LEGEND_GRADUADED]
+
+    return ret
+
 
 def gen_style_interpolate(expression, boundaries, values):
     """
@@ -274,7 +374,9 @@ def circle_boundaries_candidate(min, max):
             break
 
         # otherwise store v in the values array
-        values.append(v / scale)
+        values.append(
+            v * scale
+        )  # The the paper is false here, need mutiplication, no division
         # switch to the next base
         base_id += 1
         if base_id == len(bases):
@@ -365,7 +467,15 @@ def boundaries_round(boundaries, scale=2):
     )
 
 
-def gen_legend_circle(min, max, size, color):
+def gen_legend_circle(
+    min,
+    max,
+    size,
+    color,
+    include_no_value,
+    no_value_circle_radius,
+    no_value_color,
+):
     """
     Generate a circle legend.
     """
@@ -376,7 +486,7 @@ def gen_legend_circle(min, max, size, color):
     )
 
     r = size / math.sqrt(max / math.pi)
-    return [
+    ret = [
         {
             "diameter": math.sqrt(b / math.pi) * r,
             "boundaries": {"lower": {"value": b}},
@@ -386,11 +496,25 @@ def gen_legend_circle(min, max, size, color):
         for b in boundaries
     ]
 
+    if include_no_value:
+        ret.append(
+            {
+                "diameter": no_value_circle_radius * 2,
+                "boundaries": {"lower": {"value": None}},
+                "shape": "circle",
+                "color": no_value_color,
+            }
+        )
+
+    return ret
+
 
 def gen_layer_fill(
-    fill_color=DEFAULT_FILL_COLOR,
-    fill_opacity=DEFAULT_FILL_OPACITY,
-    stroke_color=DEFAULT_STROKE_COLOR,
+    key,
+    fill_color,
+    style,
+    include_no_value,
+    style_no_value,
 ):
     """
     Build a Mapbox GL Style layer for pylygon fill.
@@ -398,15 +522,37 @@ def gen_layer_fill(
     return {
         "type": "fill",
         "paint": {
-            "fill-color": fill_color,
-            "fill-opacity": fill_opacity,
-            "fill-outline-color": stroke_color,
+            "fill-color": get_style_no_value_condition(
+                include_no_value,
+                key,
+                fill_color,
+                style_no_value["fill_color"],
+                DEFAULT_FILL_COLOR,
+            ),
+            "fill-opacity": get_style_no_value_condition(
+                include_no_value,
+                key,
+                fill_color and style["fill_opacity"],
+                style_no_value["fill_opacity"],
+                DEFAULT_FILL_OPACITY,
+            ),
+            "fill-outline-color": get_style_no_value_condition(
+                include_no_value,
+                key,
+                fill_color and style["stroke_color"],
+                style_no_value["stroke_color"],
+                DEFAULT_STROKE_COLOR,
+            ),
         },
     }
 
 
 def gen_layer_circle(
-    radius, sort_key, fill_color, fill_opacity, stroke_color, stroke_width
+    radius,
+    sort_key,
+    style,
+    include_no_value,
+    style_no_value,
 ):
     """
     Build a Mapbox GL Style layer for circle.
@@ -415,11 +561,41 @@ def gen_layer_circle(
         "type": "circle",
         "layout": {"circle-sort-key": ["-", sort_key]},
         "paint": {
-            "circle-radius": radius,
-            "circle-color": fill_color,
-            "circle-opacity": fill_opacity,
-            "circle-stroke-color": stroke_color,
-            "circle-stroke-width": stroke_width,
+            "circle-radius": get_style_no_value_condition(
+                include_no_value,
+                radius,
+                radius,
+                style_no_value["circle_radius"],
+                DEFAULT_CIRCLE_RADIUS,
+            ),
+            "circle-color": get_style_no_value_condition(
+                include_no_value,
+                radius,
+                style["fill_color"],
+                style_no_value["fill_color"],
+                DEFAULT_FILL_COLOR,
+            ),
+            "circle-opacity": get_style_no_value_condition(
+                include_no_value,
+                radius,
+                style["fill_opacity"],
+                style_no_value["fill_opacity"],
+                DEFAULT_FILL_OPACITY,
+            ),
+            "circle-stroke-color": get_style_no_value_condition(
+                include_no_value,
+                radius,
+                style["stroke_color"],
+                style_no_value["stroke_color"],
+                DEFAULT_STROKE_COLOR,
+            ),
+            "circle-stroke-width": get_style_no_value_condition(
+                include_no_value,
+                radius,
+                style["stroke_width"],
+                style_no_value["stroke_width"],
+                DEFAULT_STROKE_WIDTH,
+            ),
         },
     }
 
@@ -433,17 +609,25 @@ def generate_style_from_wizard(layer, config):
     field = config["field"]
 
     if symbology == "graduated":
-        """ config = {
-             "field": "my_field",
-             "symbology": "graduated",
-             "boundaries": [1, 2, 3, 5],
-             "method": "equal_interval",  # How to compute boundaries if not provided
-             "fill_color": ["#ff0000", "#aa0000", "#770000", "#330000", "#000000"],
-             "fill_opacity": 0.5,
-             "stroke_color": "#ffffff",
+        """config = {
+            "field": "my_field",
+            "symbology": "graduated",
+            "boundaries": [1, 2, 3, 5],
+            "include_no_value": True,  # Show no value features on map and legend
+            "method": "equal_interval",  # How to compute boundaries if not provided
+            "style": {
+                "fill_color": ["#ff0000", "#aa0000", "#770000", "#330000", "#000000"],
+                "fill_opacity": 0.5,
+                "stroke_color": "#ffffff",
+            },
+            "no_value_style": {
+                "fill_color": "#000000",
+                "fill_opacity": 0,
+                "stroke_color": "#ffffff",
+            },
         }
         """
-        colors = config["fill_color"]
+        colors = config["style"].get("fill_color") if "style" in config else None
         if "boundaries" in config:
             boundaries = config["boundaries"]
             if len(boundaries) < 2:
@@ -455,47 +639,113 @@ def generate_style_from_wizard(layer, config):
                 'With "graduated" symbology, "boundaries" or "method" should be provided'
             )
 
-        if boundaries:
+        include_no_value = config.get("include_no_value", False)
+
+        if boundaries is not None:
+            if "style" in config:
+                config_style = {
+                    k: config["style"].get(k, DEFAULT_STYLE[k])
+                    for k in ["fill_opacity", "stroke_color"]
+                }
+            else:
+                config_style = DEFAULT_STYLE
+
+            if "no_value_style" in config:
+                config_style_no_value = {
+                    k: config["no_value_style"].get(k, DEFAULT_STYLE_NO_VALUE[k])
+                    for k in ["fill_color", "fill_opacity", "stroke_color"]
+                }
+            else:
+                config_style_no_value = DEFAULT_STYLE_NO_VALUE
+
+            field_style = get_field_style(field)
             style = gen_layer_fill(
-                fill_color=gen_style_steps(get_field_style(field), boundaries, colors),
-                fill_opacity=config.get("fill_opacity", DEFAULT_FILL_OPACITY),
-                stroke_color=config.get("stroke_color", DEFAULT_STROKE_COLOR),
+                field_style,
+                fill_color=gen_style_steps(field_style, boundaries, colors),
+                style=config_style,
+                include_no_value=include_no_value,
+                style_no_value=config_style_no_value,
             )
 
             legend_addition = {
-                "items": gen_legend_steps(boundaries, colors)[::-1],
+                "items": gen_legend_steps(
+                    boundaries,
+                    colors,
+                    include_no_value,
+                    config_style_no_value["fill_color"],
+                )[::-1],
             }
             return (style, legend_addition)
         else:
-            return (DEFAULT_STYLE_GRADUADED, {})
+            return (
+                DEFAULT_STYLE_GRADUADED,
+                {"items": [DEFAULT_LEGEND_GRADUADED]},
+            )
 
     elif symbology == "circle":
-        """ config = {
+        """config = {
             "field": "my_field",
             "symbology": "circle",
+            "include_no_value": False,  # Show no value features on map and legend
             "max_diameter": 200,
-            "fill_color": "#0000cc",
-            "fill_opacity": 0.5,
-            "stroke_color": "#ffffff",
-            "stroke_width": 1,
+            "style": {
+                "fill_color": "#0000cc",
+                "fill_opacity": 0.5,
+                "stroke_color": "#ffffff",
+                "stroke_width": 1,
+            },
+            "no_value_style": {
+                "fill_color": "#000000",
+                "fill_opacity": 0,
+                "stroke_color": "#ffffff",
+                "stroke_width": 1,
+            },
         }
         """
+        include_no_value = config.get("include_no_value", False)
+
         mm = get_positive_min_max(geo_layer, field)
-        if mm[0] is not None and mm[1] is not None:
-            mm = boundaries_round(mm)
+        if mm[1] is not None and mm[2] is not None:
+            mm = boundaries_round(mm[1:])
             boundaries = [0, math.sqrt(mm[1] / math.pi)]
             sizes = [0, config["max_diameter"] / 2]
 
             radius_base = ["sqrt", ["/", get_field_style(field), ["pi"]]]
             radius = gen_style_interpolate(radius_base, boundaries, sizes)
 
+            if "style" in config:
+                config_style = {
+                    k: config["style"].get(k, DEFAULT_STYLE[k])
+                    for k in [
+                        "fill_color",
+                        "fill_opacity",
+                        "stroke_color",
+                        "stroke_width",
+                    ]
+                }
+            else:
+                config_style = DEFAULT_STYLE
+
+            if "no_value_style" in config:
+                config_style_no_value = {
+                    k: config["no_value_style"].get(k, DEFAULT_STYLE_NO_VALUE[k])
+                    for k in [
+                        "fill_color",
+                        "fill_opacity",
+                        "stroke_color",
+                        "stroke_width",
+                        "circle_radius",
+                    ]
+                }
+            else:
+                config_style_no_value = DEFAULT_STYLE_NO_VALUE
+
             style = gen_layer_circle(
                 radius=radius,
                 sort_key=get_field_style(field),
-                fill_color=config.get("fill_color", DEFAULT_FILL_COLOR),
-                fill_opacity=config.get("fill_opacity", DEFAULT_FILL_OPACITY),
-                stroke_color=config.get("stroke_color", DEFAULT_STROKE_COLOR),
-                stroke_width=config.get("stroke_width", DEFAULT_STROKE_WIDTH),
+                style=config_style,
+                include_no_value=include_no_value,
+                style_no_value=config_style_no_value,
             )
 
             legend_addition = {
@@ -503,13 +753,19 @@ def generate_style_from_wizard(layer, config):
                     mm[0],
                     mm[1],
                     config["max_diameter"],
-                    config.get("fill_color", DEFAULT_FILL_COLOR),
+                    config_style["fill_color"],
+                    include_no_value,
+                    config_style_no_value["circle_radius"],
+                    config_style_no_value["fill_color"],
                 ),
                 "stackedCircles": True,
             }
             return (style, legend_addition)
         else:
-            return (DEFAULT_STYLE_CIRCLE, {})
+            return (
+                DEFAULT_STYLE_CIRCLE,
+                {"items": [DEFAULT_LEGEND_CIRCLE]} if include_no_value else {},
+            )
 
     else:
         raise ValueError(f'Unknow symbology "{symbology}"')
