@@ -8,7 +8,7 @@ from django.http import Http404, QueryDict
 from django.urls import reverse
 from django.utils.functional import cached_property
 from django.utils.http import urlunquote
-from django_geosource.models import WMTSSource
+from django_geosource.models import WMTSSource, FieldTypes
 
 from geostore.tokens import tiles_token_generator
 
@@ -29,13 +29,18 @@ from ..serializers import (
 from ..sources_serializers import SourceSerializer
 from ..utils import dict_merge, get_layer_group_cache_key
 
+# Map source field data_type to format_type
+TYPE_MAP = {a: b.name.lower() for a, b in dict(FieldTypes.choices()).items()}
+
 
 class SceneViewset(ModelViewSet):
     model = Scene
     queryset = Scene.objects.all()
     permission_classes = (ScenePermission,)
 
-    def get_serializer_class(self,):
+    def get_serializer_class(
+        self,
+    ):
         if self.action in ["retrieve", "update", "create", "partial_update"]:
             return SceneDetailSerializer
         return SceneListSerializer
@@ -121,7 +126,9 @@ class LayerViewset(ModelViewSet):
     def get_queryset(self):
         return self.model.objects.all()
 
-    def get_serializer_class(self,):
+    def get_serializer_class(
+        self,
+    ):
         if self.action in ["retrieve", "update", "create", "partial_update"]:
             return LayerDetailSerializer
         return LayerListSerializer
@@ -133,8 +140,7 @@ class LayerViewset(ModelViewSet):
 
 
 class LayerView(APIView):
-    """ This view generates the LayersTree used to construct the frontend
-    """
+    """This view generates the LayersTree used to construct the frontend"""
 
     permission_classes = ()
     model = Layer
@@ -182,7 +188,7 @@ class LayerView(APIView):
         return Response(response)
 
     def get_response_with_sources(self):
-        """ Return a response object containing the full layersTree with updated
+        """Return a response object containing the full layersTree with updated
         user authentication.
         """
 
@@ -214,23 +220,30 @@ class LayerView(APIView):
         ]
         return layer_structure
 
+    def get_map_settings(self, scene):
+        """Return the default map settings overridden with map settings from the scene if present"""
+        if "map_settings" in scene.config:
+            return {
+                **settings.TERRA_DEFAULT_MAP_SETTINGS,
+                **scene.config["map_settings"],
+            }
+        return settings.TERRA_DEFAULT_MAP_SETTINGS
+
     def get_layer_structure(self):
-        """ Return the structured layerTree
-        """
+        """Return the structured layerTree"""
         return {
             "title": self.scene.name,
             "type": self.scene.category,
             "layersTree": self.get_layers_tree(self.scene),
             "interactions": self.get_interactions(self.layers),
             "map": {
-                **settings.TERRA_DEFAULT_MAP_SETTINGS,
+                **self.get_map_settings(self.scene),
                 "customStyle": {"sources": [], "layers": self.get_map_layers()},
             },
         }
 
     def get_map_layers(self):
-        """ Return sources informations using serializer from sources_serializers module
-        """
+        """Return sources informations using serializer from sources_serializers module"""
         map_layers = []
         for layer in self.layers.filter(source__slug__in=self.authorized_sources):
             map_layers += [
@@ -245,15 +258,14 @@ class LayerView(APIView):
         return map_layers
 
     def get_interactions(self, layers):
-        """ Return interactions for all layers in the scene
-        """
+        """Return interactions for all layers in the scene"""
         interactions = []
         for layer in layers:
             interactions += self.get_interactions_for_layer(layer)
         return interactions
 
     def get_formatted_interactions(self, layer):
-        """ Return all interactions of a layer after beeing formatted correctly
+        """Return all interactions of a layer after beeing formatted correctly
         for the frontend
         """
         return [
@@ -274,7 +286,7 @@ class LayerView(APIView):
         ]
 
     def get_interactions_for_layer(self, layer):
-        """ Return formatted interaction of a layer
+        """Return formatted interaction of a layer
 
         It contains, popup, minisheet and custom styles
         """
@@ -324,16 +336,14 @@ class LayerView(APIView):
         return interactions
 
     def get_layers_list_for_layer(self, layer):
-        """ Return list of sublayers of a layer
-        """
+        """Return list of sublayers of a layer"""
         return [
             layer.layer_identifier,
             *[s.layer_identifier for s in layer.custom_styles.all()],
         ]
 
     def get_layers_tree(self, scene):
-        """ Return the full layer tree of a scene object
-        """
+        """Return the full layer tree of a scene object"""
         root_group = LayerGroup.objects.prefetch_related(self.prefetch_layers).get(
             view=scene, parent=None
         )
@@ -342,7 +352,7 @@ class LayerView(APIView):
         return self.get_group_dict(root_group)["layers"]
 
     def get_group_dict(self, group):
-        """ Recursive method that return the tree from a LayerGroup element.
+        """Recursive method that return the tree from a LayerGroup element.
 
         `group.settings` is injected in the group dictionnary, so any setting can be overridden.
 
@@ -413,25 +423,27 @@ class LayerView(APIView):
                 "fields": self.get_filter_fields_for_layer(layer),
                 "form": self.get_filter_forms_for_layer(layer),
             },
+            "source_credit": layer.source.credit,
         }
 
         # Set the exportable status of the layer if any filter fields is exportable
-        layer_object["filters"]["exportable"] = any(
+        layer_object["filters"]["exportable"] = layer.table_export_enable and any(
             [f["exportable"] for f in layer_object["filters"]["fields"] or []]
         )
 
         return layer_object
 
     def get_filter_fields_for_layer(self, layer):
-        """ Return the filter fields of the layer if table is enabled
-        """
+        """Return the filter fields of the layer if table is enabled"""
+
         if layer.table_enable:
             return [
                 {
                     "value": field_filter.field.name,
                     "label": field_filter.label or field_filter.field.label,
                     "exportable": field_filter.exportable,
-                    "format_type": field_filter.format_type,
+                    "format_type": field_filter.format_type
+                    or TYPE_MAP[field_filter.field.data_type],
                     "display": field_filter.display,
                     "settings": field_filter.settings,
                 }
@@ -439,8 +451,7 @@ class LayerView(APIView):
             ]
 
     def get_filter_forms_for_layer(self, layer):
-        """ Return forms of a layer if filters are enabled
-        """
+        """Return forms of a layer if filters are enabled"""
         if layer.filters_enabled:
             return [
                 {
@@ -453,8 +464,7 @@ class LayerView(APIView):
 
     @cached_property
     def authorized_sources(self):
-        """ Cached property of authorized sources from the authenticated user's groups
-        """
+        """Cached property of authorized sources from the authenticated user's groups"""
         groups = self.user_groups
         sources_slug = list(
             self.layergroup.layers.filter(
@@ -466,8 +476,7 @@ class LayerView(APIView):
 
     @cached_property
     def layers(self):
-        """ List of layers of the selected scene
-        """
+        """List of layers of the selected scene"""
         layers = (
             self.model.objects.filter(group__view=self.scene.pk)
             .order_by("order")
